@@ -1,9 +1,12 @@
-# sets the configuration parameters for NEAT
 from ConfigParser import SafeConfigParser
 import os
 
 
 class Config(object):
+    '''
+    A simple container for all of the user-configurable parameters of NEAT.
+    '''
+
     def __init__(self, filename):
         if not os.path.isfile(filename):
             raise Exception('No such config file: ' + os.path.abspath(filename))
@@ -26,17 +29,18 @@ class Config(object):
         # Genetic algorithm configuration
         self.pop_size = int(parameters.get('genetic', 'pop_size'))
         self.max_fitness_threshold = float(parameters.get('genetic', 'max_fitness_threshold'))
-        self.prob_addconn = float(parameters.get('genetic', 'prob_addconn'))
-        self.prob_addnode = float(parameters.get('genetic', 'prob_addnode'))
-        self.prob_deleteconn = float(parameters.get('genetic', 'prob_deleteconn'))
-        self.prob_deletenode = float(parameters.get('genetic', 'prob_deletenode'))
-        self.prob_mutate_bias = float(parameters.get('genetic', 'prob_mutatebias'))
+        self.prob_add_conn = float(parameters.get('genetic', 'prob_add_conn'))
+        self.prob_add_node = float(parameters.get('genetic', 'prob_add_node'))
+        self.prob_delete_conn = float(parameters.get('genetic', 'prob_delete_conn'))
+        self.prob_delete_node = float(parameters.get('genetic', 'prob_delete_node'))
+        self.prob_mutate_bias = float(parameters.get('genetic', 'prob_mutate_bias'))
         self.bias_mutation_power = float(parameters.get('genetic', 'bias_mutation_power'))
         self.prob_mutate_response = float(parameters.get('genetic', 'prob_mutate_response'))
         self.response_mutation_power = float(parameters.get('genetic', 'response_mutation_power'))
         self.prob_mutate_weight = float(parameters.get('genetic', 'prob_mutate_weight'))
+        self.prob_replace_weight = float(parameters.get('genetic', 'prob_replace_weight'))
         self.weight_mutation_power = float(parameters.get('genetic', 'weight_mutation_power'))
-        self.prob_togglelink = float(parameters.get('genetic', 'prob_togglelink'))
+        self.prob_toggle_link = float(parameters.get('genetic', 'prob_toggle_link'))
         self.elitism = float(parameters.get('genetic', 'elitism'))
 
         # genotype compatibility
@@ -48,10 +52,9 @@ class Config(object):
 
         # species
         self.species_size = int(parameters.get('species', 'species_size'))
-        # survival_threshold is the fraction for each species allowed to reproduce
         self.survival_threshold = float(parameters.get('species', 'survival_threshold'))
         self.old_threshold = int(parameters.get('species', 'old_threshold'))
         self.youth_threshold = int(parameters.get('species', 'youth_threshold'))
-        self.old_penalty = float(parameters.get('species', 'old_penalty'))  # always in (0,1)
-        self.youth_boost = float(parameters.get('species', 'youth_boost'))  # always in (1,2)
+        self.old_penalty = float(parameters.get('species', 'old_penalty'))
+        self.youth_boost = float(parameters.get('species', 'youth_boost'))
         self.max_stagnation = int(parameters.get('species', 'max_stagnation'))
