@@ -1,18 +1,35 @@
 # -*- coding: UTF-8 -*-
-import graphviz
-import matplotlib.pyplot as plt
-import numpy as np
+import warnings
+try:
+    import graphviz
+except ImportError:
+    graphviz = None
+    warnings.warn('Could not import optional dependency graphviz.')
+
+try:
+    import matplotlib.pyplot as plt
+except ImportError:
+    plt = None
+    warnings.warn('Could not import optional dependency matplotlib.')
+
+try:
+    import numpy as np
+except ImportError:
+    np = None
+    warnings.warn('Could not import optional dependency NumPy.')
 
 
-def plot_stats(stats, ylog=False, view=False, filename='avg_fitness.svg'):
+def plot_stats(best_genomes, avg_scores, ylog=False, view=False, filename='avg_fitness.svg'):
     """ Plots the population's average and best fitness. """
+    if plt is None:
+        warnings.warn("This display is not available due to a missing optional dependency (matplotlib)")
+        return
 
-    generation = [i for i in xrange(len(stats[0]))]
+    generation = range(len(best_genomes))
 
-    fitness = [c.fitness for c in stats[0]]
-    avg_pop = [avg for avg in stats[1]]
+    fitness = [c.fitness for c in best_genomes]
 
-    plt.plot(generation, avg_pop, 'b-', label="average")
+    plt.plot(generation, avg_scores, 'b-', label="average")
     plt.plot(generation, fitness, 'r-', label="best")
 
     plt.title("Population's average and best fitness")
@@ -32,6 +49,10 @@ def plot_stats(stats, ylog=False, view=False, filename='avg_fitness.svg'):
 
 def plot_spikes(spikes, view=False, filename=None):
     """ Plots the trains for a single spiking neuron. """
+    if plt is None:
+        warnings.warn("This display is not available due to a missing optional dependency (matplotlib)")
+        return
+
     plt.title("Izhikevich's spiking neuron model")
     plt.ylabel("Membrane Potential")
     plt.xlabel("Time (in ms)")
@@ -46,6 +67,10 @@ def plot_spikes(spikes, view=False, filename=None):
 
 def plot_species(species_log, view=False, filename='speciation.svg'):
     """ Visualizes speciation throughout evolution. """
+    if plt is None:
+        warnings.warn("This display is not available due to a missing optional dependency (matplotlib)")
+        return
+
     num_generations = len(species_log)
     print num_generations
     num_species = max(map(len, species_log))
@@ -74,6 +99,10 @@ def plot_species(species_log, view=False, filename='speciation.svg'):
 def draw_net(chromosome, view=False, filename=None):
     """ Receives a chromosome and draws a neural network with arbitrary topology. """
     # Attributes for network nodes.
+    if graphviz is None:
+        warnings.warn("This display is not available due to a missing optional dependency (graphviz)")
+        return
+
     node_attrs = {
         'shape': 'circle',
         'fontsize': '9',
@@ -90,11 +119,6 @@ def draw_net(chromosome, view=False, filename=None):
         'style': 'filled',
         'color': 'lightblue'}
 
-    # Attributes for disabled network connections.
-    disabled_attrs = {
-        'style': 'dotted',
-        'color': 'cornflowerblue'}
-
     dot = graphviz.Digraph(format='svg', node_attr=node_attrs)
 
     for ng_id, ng in chromosome.node_genes.items():
@@ -108,9 +132,9 @@ def draw_net(chromosome, view=False, filename=None):
     for cg in chromosome.conn_genes.values():
         a = str(cg.in_node_id)
         b = str(cg.out_node_id)
-        if cg.enabled is False:
-            dot.edge(a, b, _attributes=disabled_attrs)
-        else:
-            dot.edge(a, b)
+        style = 'solid' if cg.enabled else 'dotted'
+        color = 'green' if cg.weight > 0 else 'red'
+        width = str(0.1 + abs(cg.weight / 5.0))
+        dot.edge(a, b, _attributes={'style': style, 'color': color, 'penwidth': width})
 
-    dot.render(filename, view=view)
+    return dot.render(filename, view=view)
